use strict;
use Test::More 0.98;

use_ok $_ for qw(
    App::optex
    App::optex::Tmpfile
    App::optex::util
    App::optex::util::argv
    App::optex::util::filter
    App::optex::debug
    App::optex::help
);

done_testing;

