package App::optex;
use strict;
use warnings;

our $VERSION = "1.04";

1;

__END__

=encoding utf8

=head1 NAME

optex - optex command module directory

=head1 SYNOPSIS

B<optex> command [B<-M>I<module>] ...

or make symbolic link to B<optex> in the name of target command.

=head1 DESCRIPTION

This directory contains modules for B<optex> command.  Manual for
B<optex> command is included in the script itself.

=head1 SEE ALSO

L<github|http://kaz-utashiro.github.io/optex/>

L<Getopt::EX>

=head1 LICENSE

Copyright 2017- Kazumasa Utashiro.

You can redistribute it and/or modify it under the same terms
as Perl itself.

=head1 AUTHOR

Kazumasa Utashiro

=cut
